==> provider_software_3.8.1.tar.gz: provider_software_3.8.1/lib/biocase/archive/general.py <==
# -*- coding: utf-8 -*-
'''
$RCSfile: $
$Revision: 1632 $
$Author: j.holetschek $
$Date: 2021-05-26 11:47:49 +0200 (Mi, 26. Mai 2021) $
'''

from datetime import datetime, date
import pickle
import os, urllib.parse, unicodedata

import logging, re
log = logging.getLogger("archive.general")

# Some constants for archiving
NS_ABCD206 = 'http://www.tdwg.org/schemas/abcd/2.06'
NS_ABCD21 = 'http://www.tdwg.org/schemas/abcd/2.1'
NS_HISPID5 = 'http://www.chah.org.au/schemas/hispid/5'
ROWTYPE_DWCA = 'http://rs.tdwg.org/dwc/terms/Occurrence'

DWCA_SCHEMA_NAME = 'DwCA'

transformers = {NS_ABCD206: 'abcd2.ktr',
                NS_ABCD21: 'abcd21.ktr',
                NS_HISPID5: 'hispid5.ktr'}

ns_dataset_aware = [NS_ABCD206, NS_HISPID5, NS_ABCD21]

SCHEMA_ABCD206 = 'ABCD_2.06'
SCHEMA_ABCD21 = 'ABCD_2.1'
SCHEMA_HISPID5 = 'HISPID_5'

INV_FILENAME = 'inventory'

# Get configuration
import biocase.configuration
cfg = biocase.configuration.Cfg()


def fmt(*num):
    return tuple([re.sub(r'(\d{3})(?=\d)', r'\1,', str(n)[::-1])[::-1] if isinstance(n, int) else n for n in num])

def sizeof_fmt(num):
    for x in ['B', 'K', 'M', 'G', 'T']:
        if num < 1024:
            return "%3.0f %s" % (num, x)
        num /= 1024

def is_idle(dsa):
    '''Returns the archiving engine status for the given data source'''
    if dsa is None or dsa == '':
        raise Exception ('No datasource given.')
    else:
        idle = True
        archivePath = os.path.join(cfg.archiveWorkLocator, dsa)
        for fname in os.listdir(archivePath):
            if fname.endswith(".proc"):
                idle = False
                break
        return idle

def proc(dsa):
    '''Returns None if archiving engine is idle for the given data source, schema name being processed otherwise'''
    if dsa is None or dsa == '':
        raise Exception ('No datasource given.')
    else:
        res = None
        archivePath = os.path.join(cfg.archiveWorkLocator, dsa)
        for fname in os.listdir(archivePath):
            if fname.endswith(".proc"):
                res = fname
                break
        return res

def filenamizeDatasetTitle(title):
    ' Turns a dataset title into a string that can be used as a filename (by replacing disallowed chars)'
    if title is None:
        return None
    else:
        title = title.replace('/', '_').replace('\\', '_').replace(':', '_').replace('?', '_').replace('*', '_').replace('"', "").replace("'", "").replace('<', '_').replace('>', '_').replace('|', '_').replace('\r', '').replace('\n', '').replace('#', '').strip()
        # if not os.path.supports_unicode_filenames:
        #    title = unicodedata.normalize('NFKD', title).encode('ascii', 'ignore')
        return title


def create_xmlarchive_filename(dsa, archivePath, downloadPath, dataset, schema, schemaNS):
    'Makes up a valid xml archive filename for the specified dataset/schema'

    def makeup_filename(dataset_filenamized, schema, suffix=None):
        if suffix:
            return '%s.%s.%s.zip' % (dataset_filenamized, suffix, schema)
        else:
            return '%s.%s.zip' % (dataset_filenamized, schema)

    def archivename_exists(name):
        # Try to find the filename in the inventory
        if name in inv:
            return True

        # If not: See if it's been created in the working folder by the current archiving process;
        # if it doesn't exist, the archive name doesn't exist yet
        if os.path.exists(os.path.join(archivePath, name)):
            return True
        else:
            return False

    # First, look up the dataset in the inventory: If it already exists, return the corresponding filename. If not, go on
    inv = read_inventory(dsa)
    for aname, a in inv.items():
        if a[1] == schemaNS and a[2] == dataset:
            return aname

    # Filenamized dataset title (replace disallowed characters)
    # Path longer than allowed (260 - 5, to allow space for the suffix, if needed)? Then cut off overlength characters
    dataset_filenamized = filenamizeDatasetTitle(dataset)
    oversize = len(os.path.join(downloadPath, makeup_filename(dataset_filenamized, schema))) - 255
    if oversize > 0:
        dataset_filenamized = dataset_filenamized[:-oversize]

    # Construct full filename and append suffix if it's already existent
    # (occurs when a dataset title exists twice, with just quotes as differences)
    name = makeup_filename(dataset_filenamized, schema)
    suffix = 1
    while archivename_exists(name):
        suffix += 1
        name = makeup_filename(dataset_filenamized, schema, suffix)
    return name

def write_inventory(dsa, inventory):
    if dsa is None or dsa == '':
        raise Exception ('No datasource given.')

    fname = os.path.join(cfg.archiveWorkLocator, dsa, INV_FILENAME)
    f = open(fname, "wb")
    pickle.dump(inventory, f)
    f.close

def read_inventory(dsa):
    if dsa is None or dsa == '':
        raise Exception ('No datasource given.')

    fname = os.path.join(cfg.archiveWorkLocator, dsa, INV_FILENAME)
    if os.path.exists(fname):
        f = open(fname, "rb")
        inv = pickle.load(f)
        f.close
        # Loop through archives and remove non-existing files
        dirty = False
        for aname, dummy_ in list(inv.items()):
            if not os.path.exists(os.path.join(cfg.archiveDownloadLocator, dsa, aname)):
                del inv[aname]
                dirty = True
                log.warn('Archive file %s cannot be found; will be removed from inventory.' % os.path.join(cfg.archiveDownloadLocator, dsa, aname))
        if dirty:
            write_inventory(dsa, inv)
        # Finally, return inventory
        return inv
    else:
        return {}

def inventory_add(dsa, archives):
    '''Adds the dictionary of archives to the inventory. Each archive is defined by a dictionary item:
        key: archive filename
        value: tuple of                                                             *idx*
            - schema name, schema namespace                                         0, 1
            - dataset title (None for non-dataset aware schemas)                    2
            - records in archive, number of records dropped in archiving process    3, 4
            - file size, modification date                                          5, 6
            - source archive filename for DwCAs (None otherwise)                    7'''
    if dsa is None or dsa == '':
        raise Exception ('No datasource given.')

    inv = read_inventory(dsa)

    # update the inventory
    for name, archive in archives.items():
        # if Schema is DwC, we need to edit the archives first: find the dataset title and record counts from the source XML archive
        if archive[0] == DWCA_SCHEMA_NAME and archive[7] in inv:
            source_archive = inv[archive[7]]
            archive = (archive[0], archive[1], source_archive[2], source_archive[3], source_archive[4], archive[5], archive[6], archive[7])
        inv[name] = archive

    # write back to disk
    write_inventory(dsa, inv)

def inventory_drop(dsa, what):
    '''Removes the archive files from the inventory
    If what is a list, it contains the archives to be removed; otherwise, a schema name.'''
    if dsa is None or dsa == '':
        raise Exception ('No datasource given.')

    inv = read_inventory(dsa)

    # If list, remove only items in list; otherwise, all items of specified schema
    if type(what) is list:
        for aname in what:
            if aname in inv:
                del inv[aname]
    else:
        for aname, archive in list(inv.items()):
            if archive[0] == what:
                del inv[aname]

    # write back to disk
    write_inventory(dsa, inv)

def inventory_by_dataset(dsa):
    ' Returns a list of namespace-aware XML and DwCA archives for the given data source as a dictionary'
    ' Key: dataset title, value: list of archives, each identified by a tuple'
    if dsa is None:
        raise Exception ('No datasource given.')

    ret = {}

    inv = read_inventory(dsa)
    url_rel = cfg.archiveDownloadLocator[len(cfg.wwwLocator) + 1:].replace('\\', '/')
    baseURL = cfg.getWebappURL() + url_rel + '/' + dsa + '/'
    for aname, a in [(aname, a) for aname, a in list(inv.items()) if a[2] is not None]:
        title = a[2]
        # This is a workaround for a bug in Python 2.7 urllib.quote (http://mail.python.org/pipermail/python-dev/2006-July/067248.html)
        if isinstance(aname, str):
            aname_encoded = aname.encode('utf-8')
        else:
            aname_encoded = aname
        archive = (a[0], a[1], aname, baseURL + urllib.parse.quote(aname_encoded), a[3], a[5], datetime.fromtimestamp(a[6]))
        # Add it to the dict
        if title in ret:
            ret[title] += [archive]
        else:
            ret[title] = [archive]

    return ret

def list_archives_for_schema(dsa, schema=DWCA_SCHEMA_NAME):
    'Returns a list of all archive files for the specified schema in the download folder'
    inv = read_inventory(dsa)
    ret = []
    for aname, a in [(aname, a) for aname, a in list(inv.items()) if a[0] == schema]:
        ret += [aname]
    return ret

def inventory_summary(dsaObj):
    'Returns an overview of existing archives as a dictionary'
    'Key: schema'
    'Value: Tuple (number of archives, last modified hint)'

    # First, sort inventory by schema
    inv = {}
    for a in read_inventory(dsaObj.name).values():
        inv[a[0]] = inv.get(a[0]) + [a] if a[0] in inv else [a]

    # Loop through schemas and produce summary
    ret = {}
    for schema, archive_list in inv.items():
        cnt, mod_min, mod_max = 0, 1e10, 0
        for a in archive_list:
            cnt += 1
            mod_max = max(mod_max, a[6])
            mod_min = min(mod_min, a[6])
        smin = date.fromtimestamp(mod_min)
        smax = date.fromtimestamp(mod_max)
        ret[schema] = (str(cnt), "on %s" % (smin) if smin == smax else "between %s and %s" % (smin, smax))

    # return summary
    return ret

def drop_archives(dsa, fromdownloadfolder, what):  #schema, files=[]):
    """Drops existing archives. If from downloadfolder is true, from download folder; otherwise, from work folder
       If what is a list of file names, these files will be removed;
       if it is a schema name (string), all files of this schema will be removed."""

    # Evaluate fromdownloadfolder
    if fromdownloadfolder:
        folder = os.path.join(cfg.archiveDownloadLocator, dsa)
    else:
        folder = os.path.join(cfg.archiveWorkLocator, dsa)

    if os.path.exists(folder):
        # First, create list of files to be removed
        if type(what) is list:
            l = what
        else:
            l = [fname for fname in os.listdir(folder) if fname.endswith(what + '.zip')]

        # Remove files
        cnt = 0
        for fname in l:
            fname_full = os.path.join(folder, fname)
            if os.path.exists(fname_full):
                os.remove(fname_full)
                cnt += 1

        # And remove them from inventory, if we're working in the download folder
        if fromdownloadfolder:
            inventory_drop(dsa, what)

        # Return number of removed files
        return cnt

    else:
        raise Exception ('Folder %s doesn''t exist; file(s) could not be removed.' % folder)


#===============================================================================
# def list_archives(dsaObj, datasetaware_only=True):
#    ' Returns a list of archive files for the given data source as a dictionary'
#    ' Key: filenamized dataset title'
#    ' Value: list of archives, each being a tuple (schema, NS, file name, archive URL, modified date, size, records, dropped). NS is None for DwCA'
#    if dsaObj is None:
#        raise Exception ('No datasource given.')
#    else:
#        # Get the namespace aware schema names, add DwC
#        schemas = [(os.path.splitext(schema.name)[0], schema.NS) for schema in dsaObj.getSchemaListObj().getSchemaList()]
#        schemas.append((DWCA_SCHEMA_NAME, None))
#        dldPath = os.path.join(cfg.archiveDownloadLocator, dsaObj.name)
#        baseURL = cfg.getWebappURL() + cfg.archiveDownloadLocator[len(cfg.wwwLocator) + 1:].replace('\\', '/') + '/' + dsaObj.name + '/'
#
#        ret = {}
#        if os.path.exists(dldPath):
#            l = [fname for fname in os.listdir(dldPath) if os.path.splitext(fname)[1] == '.zip' ]
#            for fname in l:
#                fname_head, _ext = os.path.splitext(fname)
#                # Try to find it in the list of dataset aware schema
#                title, NS = None, None
#                for sname, sNS in schemas:
#                    if fname_head.endswith(sname):
#                        title = fname_head[:-(len(sname) + 1)]
#                        NS = sNS
#                        schema = sname
#                if title == '' and not datasetaware_only:
#                    # Dataset-unaware schema: Set  title to schema name if all archives are to be listed
#                    title = schema
#                if title is not None:
#                    stat = os.stat(os.path.join(dldPath, fname))
#                    modified = datetime.fromtimestamp(stat.st_mtime)
#                    size = sizeof_fmt(stat.st_size)
#                    archive = (schema, NS, fname, baseURL + fname, modified, size)
#                    # Add it to the dict
#                    if ret.has_key(title):
#                        ret[title] += [archive]
#                    else:
#                        ret[title] = [archive]
#
#        return ret
#===============================================================================

==> provider_software_3.8.4.tar.gz: provider_software_3.8.4/lib/biocase/archive/general.py <==
# -*- coding: utf-8 -*-
'''
$RCSfile: $
$Revision: 1656 $
$Author: j.holetschek $
$Date: 2021-12-21 11:11:35 +0100 (Di, 21. Dez 2021) $
'''

from datetime import datetime, date
import pickle
import os, urllib.parse, unicodedata

import logging, re
log = logging.getLogger("archive.general")

# Some constants for archiving
NS_ABCD206 = 'http://www.tdwg.org/schemas/abcd/2.06'
NS_ABCD21 = 'http://www.tdwg.org/schemas/abcd/2.1'
NS_HISPID5 = 'http://www.chah.org.au/schemas/hispid/5'
ROWTYPE_DWCA = 'http://rs.tdwg.org/dwc/terms/Occurrence'

DWCA_SCHEMA_NAME = 'DwCA'

transformers = {NS_ABCD206: 'abcd2.ktr',
                NS_ABCD21: 'abcd21.ktr',
                NS_HISPID5: 'hispid5.ktr'}

ns_dataset_aware = [NS_ABCD206, NS_HISPID5, NS_ABCD21]

SCHEMA_ABCD206 = 'ABCD_2.06'
SCHEMA_ABCD21 = 'ABCD_2.1'
SCHEMA_HISPID5 = 'HISPID_5'

INV_FILENAME = 'inventory'

# Get configuration
import biocase.configuration
cfg = biocase.configuration.Cfg()


def fmt(*num):
    return tuple([re.sub(r'(\d{3})(?=\d)', r'\1,', str(n)[::-1])[::-1] if isinstance(n, int) else n for n in num])

def sizeof_fmt(num):
    for x in ['B', 'K', 'M', 'G', 'T']:
        if num < 1024:
            return "%3.0f %s" % (num, x)
        num /= 1024

def is_idle(dsa):
    '''Returns the archiving engine status for the given data source'''
    if dsa is None or dsa == '':
        raise Exception ('No datasource given.')
    else:
        idle = True
        archivePath = os.path.join(cfg.archiveWorkLocator, dsa)
        for fname in os.listdir(archivePath):
            if fname.endswith(".proc"):
                idle = False
                break
        return idle

def proc(dsa):
    '''Returns None if archiving engine is idle for the given data source, schema name being processed otherwise'''
    if dsa is None or dsa == '':
        raise Exception ('No datasource given.')
    else:
        res = None
        archivePath = os.path.join(cfg.archiveWorkLocator, dsa)
        for fname in os.listdir(archivePath):
            if fname.endswith(".proc"):
                res = fname
                break
        return res

def filenamizeDatasetTitle(title):
    ' Turns a dataset title into a string that can be used as a filename (by replacing disallowed chars)'
    if title is None:
        return None
    else:
        title = title.replace('/', '_').replace('\\', '_').replace(':', '_').replace('?', '_').replace('*', '_').replace('"', "").replace("'", "").replace('<', '_').replace('>', '_').replace('|', '_').replace('\r', '').replace('\n', '').replace('#', '').strip()
        if not os.path.supports_unicode_filenames:
            title = unicodedata.normalize('NFKD', title).encode('ascii', 'ignore').decode('ascii')
        return title


def create_xmlarchive_filename(dsa, archivePath, downloadPath, dataset, schema, schemaNS):
    'Makes up a valid xml archive filename for the specified dataset/schema'

    def makeup_filename(dataset_filenamized, schema, suffix=None):
        if suffix:
            return '%s.%s.%s.zip' % (dataset_filenamized, suffix, schema)
        else:
            return '%s.%s.zip' % (dataset_filenamized, schema)

    def archivename_exists(name):
        # Try to find the filename in the inventory
        if name in inv:
            return True

        # If not: See if it's been created in the working folder by the current archiving process;
        # if it doesn't exist, the archive name doesn't exist yet
        if os.path.exists(os.path.join(archivePath, name)):
            return True
        else:
            return False

    # First, look up the dataset in the inventory: If it already exists, return the corresponding filename. If not, go on
    inv = read_inventory(dsa)
    for aname, a in inv.items():
        if a[1] == schemaNS and a[2] == dataset:
            return aname

    # Filenamized dataset title (replace disallowed characters)
    # Path longer than allowed (260 - 5, to allow space for the suffix, if needed)? Then cut off overlength characters
    dataset_filenamized = filenamizeDatasetTitle(dataset)
    oversize = len(os.path.join(downloadPath, makeup_filename(dataset_filenamized, schema))) - 255
    if oversize > 0:
        dataset_filenamized = dataset_filenamized[:-oversize]

    # Construct full filename and append suffix if it's already existent
    # (occurs when a dataset title exists twice, with just quotes as differences)
    name = makeup_filename(dataset_filenamized, schema)
    suffix = 1
    while archivename_exists(name):
        suffix += 1
        name = makeup_filename(dataset_filenamized, schema, suffix)
    return name

def write_inventory(dsa, inventory):
    if dsa is None or dsa == '':
        raise Exception ('No datasource given.')

    fname = os.path.join(cfg.archiveWorkLocator, dsa, INV_FILENAME)
    f = open(fname, "wb")
    pickle.dump(inventory, f)
    f.close

def read_inventory(dsa):
    if dsa is None or dsa == '':
        raise Exception ('No datasource given.')

    fname = os.path.join(cfg.archiveWorkLocator, dsa, INV_FILENAME)
    if os.path.exists(fname):
        f = open(fname, "rb")
        inv = pickle.load(f)
        f.close
        # Loop through archives and remove non-existing files
        dirty = False
        for aname, dummy_ in list(inv.items()):
            if not os.path.exists(os.path.join(cfg.archiveDownloadLocator, dsa, aname)):
                del inv[aname]
                dirty = True
                log.warn('Archive file %s cannot be found; will be removed from inventory.' % os.path.join(cfg.archiveDownloadLocator, dsa, aname))
        if dirty:
            write_inventory(dsa, inv)
        # Finally, return inventory
        return inv
    else:
        return {}

def inventory_add(dsa, archives):
    '''Adds the dictionary of archives to the inventory. Each archive is defined by a dictionary item:
        key: archive filename
        value: tuple of                                                             *idx*
            - schema name, schema namespace                                         0, 1
            - dataset title (None for non-dataset aware schemas)                    2
            - records in archive, number of records dropped in archiving process    3, 4
            - file size, modification date                                          5, 6
            - source archive filename for DwCAs (None otherwise)                    7'''
    if dsa is None or dsa == '':
        raise Exception ('No datasource given.')

    inv = read_inventory(dsa)

    # update the inventory
    for name, archive in archives.items():
        # if Schema is DwC, we need to edit the archives first: find the dataset title and record counts from the source XML archive
        if archive[0] == DWCA_SCHEMA_NAME and archive[7] in inv:
            source_archive = inv[archive[7]]
            archive = (archive[0], archive[1], source_archive[2], source_archive[3], source_archive[4], archive[5], archive[6], archive[7])
        inv[name] = archive

    # write back to disk
    write_inventory(dsa, inv)

def inventory_drop(dsa, what):
    '''Removes the archive files from the inventory
    If what is a list, it contains the archives to be removed; otherwise, a schema name.'''
    if dsa is None or dsa == '':
        raise Exception ('No datasource given.')

    inv = read_inventory(dsa)

    # If list, remove only items in list; otherwise, all items of specified schema
    if type(what) is list:
        for aname in what:
            if aname in inv:
                del inv[aname]
    else:
        for aname, archive in list(inv.items()):
            if archive[0] == what:
                del inv[aname]

    # write back to disk
    write_inventory(dsa, inv)

def inventory_by_dataset(dsa):
    ' Returns a list of namespace-aware XML and DwCA archives for the given data source as a dictionary'
    ' Key: dataset title, value: list of archives, each identified by a tuple'
    if dsa is None:
        raise Exception ('No datasource given.')

    ret = {}

    inv = read_inventory(dsa)
    url_rel = cfg.archiveDownloadLocator[len(cfg.wwwLocator) + 1:].replace('\\', '/')
    baseURL = cfg.getWebappURL() + url_rel + '/' + dsa + '/'
    for aname, a in [(aname, a) for aname, a in list(inv.items()) if a[2] is not None]:
        title = a[2]
        # This is a workaround for a bug in Python 2.7 urllib.quote (http://mail.python.org/pipermail/python-dev/2006-July/067248.html)
        if isinstance(aname, str):
            aname_encoded = aname.encode('utf-8')
        else:
            aname_encoded = aname
        archive = (a[0], a[1], aname, baseURL + urllib.parse.quote(aname_encoded), a[3], a[5], datetime.fromtimestamp(a[6]))
        # Add it to the dict
        if title in ret:
            ret[title] += [archive]
        else:
            ret[title] = [archive]

    return ret

def list_archives_for_schema(dsa, schema=DWCA_SCHEMA_NAME):
    'Returns a list of all archive files for the specified schema in the download folder'
    inv = read_inventory(dsa)
    ret = []
    for aname, a in [(aname, a) for aname, a in list(inv.items()) if a[0] == schema]:
        ret += [aname]
    return ret

def inventory_summary(dsaObj):
    'Returns an overview of existing archives as a dictionary'
    'Key: schema'
    'Value: Tuple (number of archives, last modified hint)'

    # First, sort inventory by schema
    inv = {}
    for a in read_inventory(dsaObj.name).values():
        inv[a[0]] = inv.get(a[0]) + [a] if a[0] in inv else [a]

    # Loop through schemas and produce summary
    ret = {}
    for schema, archive_list in inv.items():
        cnt, mod_min, mod_max = 0, 1e10, 0
        for a in archive_list:
            cnt += 1
            mod_max = max(mod_max, a[6])
            mod_min = min(mod_min, a[6])
        smin = date.fromtimestamp(mod_min)
        smax = date.fromtimestamp(mod_max)
        ret[schema] = (str(cnt), "on %s" % (smin) if smin == smax else "between %s and %s" % (smin, smax))

    # return summary
    return ret

def drop_archives(dsa, fromdownloadfolder, what):  #schema, files=[]):
    """Drops existing archives. If from downloadfolder is true, from download folder; otherwise, from work folder
       If what is a list of file names, these files will be removed;
       if it is a schema name (string), all files of this schema will be removed."""

    # Evaluate fromdownloadfolder
    if fromdownloadfolder:
        folder = os.path.join(cfg.archiveDownloadLocator, dsa)
    else:
        folder = os.path.join(cfg.archiveWorkLocator, dsa)

    if os.path.exists(folder):
        # First, create list of files to be removed
        if type(what) is list:
            l = what
        else:
            l = [fname for fname in os.listdir(folder) if fname.endswith(what + '.zip')]

        # Remove files
        cnt = 0
        for fname in l:
            fname_full = os.path.join(folder, fname)
            if os.path.exists(fname_full):
                os.remove(fname_full)
                cnt += 1

        # And remove them from inventory, if we're working in the download folder
        if fromdownloadfolder:
            inventory_drop(dsa, what)

        # Return number of removed files
        return cnt

    else:
        raise Exception ('Folder %s doesn''t exist; file(s) could not be removed.' % folder)


#===============================================================================
# def list_archives(dsaObj, datasetaware_only=True):
#    ' Returns a list of archive files for the given data source as a dictionary'
#    ' Key: filenamized dataset title'
#    ' Value: list of archives, each being a tuple (schema, NS, file name, archive URL, modified date, size, records, dropped). NS is None for DwCA'
#    if dsaObj is None:
#        raise Exception ('No datasource given.')
#    else:
#        # Get the namespace aware schema names, add DwC
#        schemas = [(os.path.splitext(schema.name)[0], schema.NS) for schema in dsaObj.getSchemaListObj().getSchemaList()]
#        schemas.append((DWCA_SCHEMA_NAME, None))
#        dldPath = os.path.join(cfg.archiveDownloadLocator, dsaObj.name)
#        baseURL = cfg.getWebappURL() + cfg.archiveDownloadLocator[len(cfg.wwwLocator) + 1:].replace('\\', '/') + '/' + dsaObj.name + '/'
#
#        ret = {}
#        if os.path.exists(dldPath):
#            l = [fname for fname in os.listdir(dldPath) if os.path.splitext(fname)[1] == '.zip' ]
#            for fname in l:
#                fname_head, _ext = os.path.splitext(fname)
#                # Try to find it in the list of dataset aware schema
#                title, NS = None, None
#                for sname, sNS in schemas:
#                    if fname_head.endswith(sname):
#                        title = fname_head[:-(len(sname) + 1)]
#                        NS = sNS
#                        schema = sname
#                if title == '' and not datasetaware_only:
#                    # Dataset-unaware schema: Set  title to schema name if all archives are to be listed
#                    title = schema
#                if title is not None:
#                    stat = os.stat(os.path.join(dldPath, fname))
#                    modified = datetime.fromtimestamp(stat.st_mtime)
#                    size = sizeof_fmt(stat.st_size)
#                    archive = (schema, NS, fname, baseURL + fname, modified, size)
#                    # Add it to the dict
#                    if ret.has_key(title):
#                        ret[title] += [archive]
#                    else:
#                        ret[title] = [archive]
#
#        return ret
#===============================================================================
